(function ($) {
    /*****************************Front Form *******************/
    $(document).bind('erf_process_form', function (ev, selector, submission) {
        var erfContainer = selector || $(".erf-container");
        var formContainer = erfContainer.find('.erf-front-form');

        if (formContainer.length == 0)
            return;

        var parsleyConfig = {
            errorsContainer: function (pEle) {
                var $err = pEle.$element.closest('.form-group');
                return $err;
            },
        }
        window.Parsley.addValidator('confirmPassword', {
            validateString: function (value, passwordFieldId) {
                return $("#" + passwordFieldId).val() == value;
            },
            messages: {
                en: erf_global.parsley.confirmPassword,
            }
        });

        if (erf_global.logged_in == 1) {
            var primaryFields = formContainer.find('[type=user_email],[type=password],[entity-type=user][entity-property=username]');
            primaryFields.val('');
            primaryFields.attr('disabled', true);
            primaryFields.attr('placeholder', '');
            primaryFields.addClass('erf-disabled');
            formContainer.find("[user_roles=true]").attr('disabled', true);
        }

        parsleyConfig.excluded = 'input:hidden,select:hidden,textarea:hidden,file:hidden,:disabled,.erf-disabled';

        var twoColumnLayout = function (form, selector) {
            form.find(selector).each(function () {
                var page = $(this);
                var pageElements = page.children('div');
                var twoColumnWrapper = $('<div class="erf-two-columns" />');
                var row = [];
                for (i = 0; i < pageElements.length; i++) {
                    var field = $(pageElements[i]);
                    var fieldNext = $(pageElements[i + 1]);

                    if (field.hasClass('form-group') && fieldNext.length > 0 && fieldNext.hasClass('form-group')) {
                        pageElements.filter(':eq(' + i + '),:eq(' + (i + 1) + ')').wrapAll(twoColumnWrapper);
                        i++;
                    } else {
                        field.wrap(twoColumnWrapper);
                    }
                }

            });
        }

        var getFormLayout = function (form) {
            var formContainer = form.closest('.erf-container');
            if (formContainer.hasClass('erf-layout-two-column'))
            {
                return 2;
            }
            return 1;

        }


        submit_form = function (form) {
            // Remove all previous error
            $(form).find('[custom-type=page-break]').removeClass('erf-has-errors');

            var formData = new FormData(form);
            $(form).find('.erf-errors').html('');
            var formParentBlock = $(form).closest('.erf-container');
            formParentBlock.find('.erf-field-error').remove();
            var submitButton = $(form).find('.erf-submit-button button');
            if (submitButton.length == 0) { // In case of multipage form
                submitButton = $(form).find('.erf-form-nav :submit');
            }

            submitButton.append('<span class="erf-loader"></span>');
            submitButton.attr('disabled', true);
            $.ajax({
                url: erf_global.ajax_url,
                type: 'POST',
                data: formData,
                async: true,
                success: function (response) {
                    submitButton.attr('disabled', false);
                    submitButton.find('.erf-loader').remove();
                    // Check for reCaptcha elements
                    var recap = $(form).find('.erf-recaptcha');
                    if (recap.length > 0 && typeof grecaptcha !== 'undefined' && grecaptcha && grecaptcha.reset) {
                        grecaptcha.reset();
                    }
                    try {
                        erf_handle_form_ajax_response(response, selector, form);
                    } catch (ex) {
                    }
                },
                cache: false,
                contentType: false,
                processData: false
            });
        }

        /*
         * Multipage functionality
         */
        var paginateForm = function (form) {
            var navContainer = form.find('.erf-form-nav'); // It holds previous button
            var pages = []; // Holds all the pages. Note: There are no HTML sections to hold page elements.
            var pageBreaks = []; // Elements which defines page break
            var submitButton = form.find('.erf-submit-button button');
            var submitButtonTitle = submitButton.html();
            var nextButton;
            var prevButton;
            var hideTabs = false;

            // Adds index to each input element so that Parsley can validate them in individual group.
            var addElementGroup = function (formElements, index) {
                formElements.attr('data-parsley-group', 'block-' + index);
                formElements.find(':input').attr('data-parsley-group', 'block-' + index);
            }

            // Ads and registers previous button 
            var addNavButtons = function (index) {
                navContainer.html('');
                // Creating nav buttons
                submitButton.removeClass('erf-nav-next');
                prevButton = submitButton.clone();
                prevButton.attr('type', 'button');
                prevButton.addClass('erf-nav-prev');
                prevButton.html(erf_global.js_strings.prev);
                nextButton = submitButton.clone();
                nextButton.html(erf_global.js_strings.next);
                nextButton.addClass('erf-nav-next');

                prevButton.click(function () {
                    var selectedIndex = parseInt(navContainer.attr('current-page-index')) - 1;
                    goToPage(selectedIndex);
                });
                if (index > 0)  // Do not add Previous button on first Page
                    navContainer.append(prevButton);
                navContainer.append(nextButton);
                if (index == (pages.length - 1)) {
                    nextButton.html(submitButtonTitle);
                }
                submitButton.remove();
            }

            // Remove Page Break closest DIV
            form.find('div[custom-type=page-break]').unwrap();

            // Check if first element is page Break.
            var firstPageBreak = form.find('.rendered-form div').first().attr('custom-type');
            if (firstPageBreak != 'page-break')  // First element is not page break. 
            {
                form.find('.rendered-form').prepend('<div custom-type="page-break" class="page-break">Page Break</div>');
                hideTabs = true; // Hides tab system for Page breaks
            }

            var pageBreaks = form.find('div[custom-type=page-break]');
            pageBreaks.addClass('page-break');
            navContainer.attr('current-page-index', 0);   // Assigning current page index    
            if (pageBreaks.length > 1) {
                form.find('.erf-external-form-elements').hide(); // Hiding external form elements for multipage forms
            }


            var goToPage = function (selectedIndex, next) {
                var currentPageIndex = form.find('.active-page').attr('page-index');
                var next = next || false;
                if (next || selectedIndex < currentPageIndex) {
                    for (i = 0; i < pages.length; i++) {
                        if (selectedIndex == i)
                        {
                            pages[i].show();
                            navContainer.attr('current-page-index', selectedIndex);
                            pageBreaks.removeClass('active-page');
                            $(pageBreaks[selectedIndex]).addClass('active-page');
                            navContainer.html('');
                        } else
                            pages[i].hide();
                    }
                    addNavButtons(selectedIndex);
                }

                if ((selectedIndex + 1) == pageBreaks.length) {
                    form.find('.erf-external-form-elements').show(); // Showing external form elements at last pagination
                } else {
                    form.find('.erf-external-form-elements').hide();
                }
                // Scroll on top
                $('html, body').animate({
                    scrollTop: form.closest('.erf-container').offset().top
                }, 300);
            }
            if (hideTabs || pageBreaks.length == 1) {
                pageBreaks.hide();
            }
            if (pageBreaks.length > 1) {
                pageBreaks.each(function (index) {
                    var formElements = $(this).nextUntil('div[custom-type=page-break]');
                    if (formElements.length == 0) { // Inserting blank page
                        formElements = $('<div class="form-group">&nbsp;</div>');
                    }
                    formElements.wrapAll('<div class="erf-page erf-page-' + index + '"></div>');
                    pages.push(formElements);
                    addElementGroup(formElements, index);
                    if (index > 0)
                        formElements.hide(); // Be default hiding all the page elements except initial page
                });
            }


            if (pages.length > 1) {


                form.attr('erf-multipage', 1); // Adding multipage flag

                // By default show first page elements
                pages[0].show();

                // Adding index properties to each page break, Appends page breaks in form starting
                for (i = 0; i < pageBreaks.length; i++) {
                    $(pageBreaks[i]).attr('page-index', i);
                    if (i == 0) {
                        $(pageBreaks[i]).addClass('active-page');
                    }
                    form.find('.rendered-form').before(pageBreaks[i]);
                }
                pageBreaks.wrapAll("<div class='erf-page-breaks'></div>");

                // Registers click event to allow jumping to previous pages
                pageBreaks.click(function () {
                    var selectedIndex = $(this).attr('page-index');
                    goToPage(selectedIndex);
                });

                var formLayout = getFormLayout(form);
                if (formLayout == 2)
                {
                    twoColumnLayout(form, '.erf-page');
                }


                /*
                 * Binds submit button.
                 * Hides previous page elements and shows new elements (If any)
                 * Registers parsley validation.
                 * Sends ajax request on successfull validation.
                 */
                $(form).submit(function (event) {
                    event.preventDefault();
                    var currentIndex = parseInt(navContainer.attr('current-page-index'));
                    var nextIndex = currentIndex + 1;
                    var formInstance = form.parsley(parsleyConfig);
                    pageBreaks.removeClass('erf-has-errors');
                    if (pages[nextIndex] !== void 0) {
                        goToPage(nextIndex, true);
                    }
                    formInstance.whenValidate({
                        group: 'block-' + currentIndex
                    }).done(function () {  // Triggers for last page.
                        if (nextIndex == pages.length) {
                            submit_form(form[0]);
                        }
                    });

                });
            }
        }
        // Multipage functionality ends here


        formContainer.each(function () {
            var formInstance = $(this).parsley(parsleyConfig);
            Parsley.addMessages('en', erf_global.parsley);

            // Cloning submit button at last to add any elements externally in Form and delete auto generated button
            var submitBtnBlock = $(this).find('.erf-submit-button');
            var genratedButtonBlock = $(this).find('[type=submit]').closest('.form-group ');

            if (genratedButtonBlock.length > 0) {
                genratedButtonBlock.clone().appendTo(submitBtnBlock);
                genratedButtonBlock.remove();
            }


            var form = $(this);
            var formLayout = getFormLayout(form);
            var formGroups = form.find('.rendered-form');

            /* Change user_email type to email */
            var userEmailField = form.find('input[type=user_email]');
            if (userEmailField.length > 0) {
                userEmailField.attr('data-parsley-type', "email");
                userEmailField.attr('type', "email");
            }

            /*
             * Initialize datepicker for each of the date field.
             */
            form.find('[data-erf-type=date]').each(function () {
                var minDate = $(this).attr('min');
                var maxDate = $(this).attr('max');
                var dateFormat = $(this).data('date-format');
                var dateConfig = {dateFormat: dateFormat, changeMonth: true, changeYear: true, yearRange: '-100:+20'};
                if (minDate)
                    dateConfig.minDate = new Date(minDate);
                if (maxDate)
                    dateConfig.maxDate = new Date(maxDate);
                $(this).datepicker(dateConfig);
                $(this).attr('type', 'text');
                $(this).removeAttr('min'); // Removing min attribute to disable default field validation
                $(this).removeAttr('max');  // Removing max attribute to disable default field validation
            });


            // Current URL for after login redirect (Only for registration forms)
            form.find('#erform_redirect_to').val(jQuery(location).attr('href'));

            // Button position
            form.find('button[data-erf-btn-pos]').each(function () {
                var positionName = $(this).data('erf-btn-pos');
                if (positionName) {
                    $(this).closest('.fb-button').addClass('erf-btn-' + positionName);
                }

            });

            /* Payment related */
            var paymentWrapper = form.find('.erf-payment-wrapper');
            if (paymentWrapper.length > 0) {
                formGroups.append(paymentWrapper.clone(true, true).html());
                paymentWrapper.remove();

                var getPrice = function (element) {
                    var priceValue = 0;
                    var planType = element.data('plan-type');
                    if (planType == '')
                        return;
                    if (planType == 'user') {
                        priceValue = parseFloat(element.val());
                        if (isNaN(priceValue)) {
                            return 0;
                        }
                        priceValue = priceValue;
                    } else if (planType == 'product') {
                        if (element.is(':checked')) {
                            priceValue = parseFloat(element.data('erf-price'));
                            if (isNaN(priceValue)) {
                                return 0;
                            }
                            priceValue = priceValue;
                        }
                    }

                    return priceValue;
                }

                var updatePrieHTML = function (finalPrice) {
                    var totalPaymentContainer = form.find('.erf-total-payment:first');
                    totalPaymentContainer.html(finalPrice.round(2));
                    if (finalPrice <= 0) {
                        form.find('.erf-payment-methods').slideUp();
                    } else
                    {
                        form.find('.erf-payment-methods').slideDown();
                    }
                }
                form.find('.erf-price').change(function () {
                    var finalPrice = 0;
                    var tempPrice = 0;

                    $('.erf-price').each(function () { // Calculating price from each plan field
                        tempPrice = getPrice($(this));
                        if (tempPrice < 0) {
                            tempPrice *= -1;
                        }
                        finalPrice += tempPrice;
                    });
                    updatePrieHTML(finalPrice);
                });

                form.find('input.erf-price').on('keyup blur', function () {
                    var finalPrice = 0;
                    var tempPrice = 0;
                    $('.erf-price').each(function () { // Calculating price from each plan field
                        tempPrice = getPrice($(this));
                        if (tempPrice < 0) {
                            tempPrice *= -1;
                        }
                        finalPrice += tempPrice;
                    });
                    updatePrieHTML(finalPrice);
                });

            }
            /* Payment ends here */

            paginateForm(form);
            var is_multipage = form.attr('erf-multipage');
            if (is_multipage != 1) {
                if (formLayout == 2) // Check if two column layout
                {
                    twoColumnLayout(form, '.rendered-form');
                }

                $(this).submit(function (event) {
                    event.preventDefault();
                    submit_form($(this)[0]);
                });
            } else
            {
                submitBtnBlock.find('button').html(erf_global.js_strings.next);
                submitBtnBlock.find('button').addClass('erf-nav-next erf-submit-button');
            }



            // Procssing hook for each form.
            var formID = form.data('erf-form-id');
            var submissionID = form.data('erf-submission-id');
            var submission = null;
            var formData = null;
            // Fetch submission data
            if (formID > 0) {
                $.post(erf_global.ajax_url, {form_id: formID, submission_id: submissionID, action: 'erforms_form_submission_data'}, function (res) {
                    if (res.success) {
                        if (res.data.hasOwnProperty('submission')) {
                            submission = res.data.submission;
                        }
                        if (res.data.hasOwnProperty('form')) {
                            formData = res.data.form;
                        }
                        var userMeta;
                        if (res.data.hasOwnProperty('user_meta')) {
                            userMeta = res.data.user_meta;
                            prefilUsermeta(userMeta, form);  // Prefil user meta initially.
                        }
                        $(document).trigger('erf_process_form_conditions', [form, submission, formData]);
                        $(document).trigger('erf_process_form_dynamic_fields', [form, submission, formData, userMeta]);
                        $(document).trigger('erf_edit_submission_form', [form, submission, formData]);
                    }
                }).fail(function (xhr, textStatus, e) {
                });
            }
        });


        /*
         * Handling Other Option for Checkbox and Radio Buttons
         */
        formContainer.find('.other-val').hide();
        formContainer.find('input.other-option').each(function () {

            if ($(this).is(':radio')) { // Radio Button handling
                var radioGroupName = $(this).attr('name');
                var parentGroup = $(this).closest('.radio-group');
                var otherElementId = '#' + radioGroupName + '-other-value';
                formContainer.find("[name='" + radioGroupName + "']").change(function () {
                    if ($(this).hasClass('other-option')) {
                        parentGroup.find(otherElementId).slideDown();
                        parentGroup.find(otherElementId).keyup(function () {
                            parentGroup.find('.other-option').val($(this).val());
                        });
                    } else {
                        parentGroup.find(otherElementId).slideUp();
                    }
                });
            } else {
                $(this).change(function () {
                    var self = $(this);
                    var otherElement = self.siblings('label').children('.other-val');
                    if (otherElement.length == 0)
                        return;

                    if ($(this).is(':checked')) {
                        otherElement.slideDown();
                        otherElement.keyup(function () {
                            self.val(otherElement.val());
                        });
                        return;
                    }
                    otherElement.slideUp();
                });
            }

        });
        // Other option handling ended. 

        formContainer.find('input[masking]').each(function () {
            var pattern = $(this).attr('masking');
            if (pattern) {
                var target = this;
                $(target).mask(pattern);
            }
        });

        formContainer.find('div[custom-type=spacer]').each(function () {
            var height = $(this).attr('height');
            $(this).html('');
            if (height) {
                $(this).css('height', height + 'px');
            }
        });



        /*
         * Used for Front form (For admin only)
         */
        var erforms_change_form_layout = function (form) {
            var formData = new FormData(form);
            $.ajax({
                url: erf_global.ajax_url,
                type: 'POST',
                data: formData,
                async: false,
                success: function (response) {
                    try {
                        if (response.success)
                        {
                            location.reload();
                        }
                    } catch (ex) {
                    }
                },
                cache: false,
                contentType: false,
                processData: false
            });
        }

    });
    var prefilUsermeta = function (data, form) {
        for (var key in data) {
            var metaVal = data[key];
            var field = $("[name=" + key + "]");
            if (field.length == 0) {
                field = $("[name='" + key + "[]']");
                if (field.length == 0)
                    continue;
            }
            if (field.is(':checkbox')) {
                if (metaVal instanceof Array) {
                    field.each(function () {
                        $(this).prop("checked", ($.inArray($(this).val(), metaVal) != -1));
                    });
                }
            } else if (field.is(':file')) {
                continue;
            } else if (field.is(':radio')) {
                field.filter('[value="' + metaVal + '"]').attr('checked', true);
            } else {
                field.val(metaVal);
            }
            field.trigger('change');
        }
    }
    /*Dynamic Field Population */
    $(document).bind('erf_process_form_dynamic_fields', function (ev, form, submission, formData, userMeta) {
        var formId = form.find('[name=erform_id]').val();

        if (formId.length > 0) {
            $.post(erf_global.ajax_url, {form_id: formId, action: 'erforms_get_form_meta', meta: 'dynamic_rules'}, function (res) {
                if (res.success) {
                    var rules = res.data.dynamic_rules;
                    if (rules.load.length > 0) {
                        fieldCommandsOnLoad();
                    }

                    if (rules.change.length > 0) {
                        fieldCommandOnChange(rules.change);
                    }
                    applyConfiguration(res.data);
                }
            }).fail(function (xhr, textStatus, e) {
                //console.log(xhr.responseText);
            });
        }


        var applyConfiguration = function (data) {

            // Mobile field international telephone configuration
            if (data.hasOwnProperty('tel_config')) {
                var tels = form.find('[type=tel][enable-intl=true]');
                if (tels.length > 0) {
                    tels.each(function () {
                        data.tel_config.hiddenInput = $(this).attr('name') + "-intl";
                        data.tel_config.geoIpLookup = window[data.tel_config.geoIpLookup];
                        $(this).intlTelInput(data.tel_config);
                        var dialCode = '';
                        $(this).bind('keyup change input', function () {
                            dialCode = $(this).intlTelInput("getSelectedCountryData").dialCode;
                            if (dialCode)
                                form.find('[name="' + data.tel_config.hiddenInput + '"]').val($(this).intlTelInput('getNumber'));
                        });
                        $(this).on("countrychange", function () {
                            form.find('[name="' + data.tel_config.hiddenInput + '"]').val($(this).intlTelInput('getNumber'));
                        });
                        var currentEl = $(this);
                        dialCode = $(this).intlTelInput("getSelectedCountryData").dialCode;
                        if (dialCode) {
                            setTimeout(function () {
                                form.find('[name="' + data.tel_config.hiddenInput + '"]').val(currentEl.intlTelInput("getNumber"));
                            }, 1000);
                        }
                    });
                }
            }
            //Tel configuration ends here 

            // Password field configuration
            if (data.hasOwnProperty('pass_config')) {
                form.find('[type=password]').each(function () {
                    $(this).password(data.pass_config);
                });
            }
        }

        var fieldCommandsOnLoad = function (field_name) {
            var data = {action: 'erforms_field_load_commands', form_id: formId};
            $.post(erf_global.ajax_url, data, function (res) {
                if (res.success) {
                    var commands = res.data.commands;
                    var dynamicFields = [];
                    for (var i = 0; i < commands.length; i++) {
                        var command = commands[i];
                        for (var j = 0; j < command.on.length; j++) {
                            var field_name = command.on[j];
                            dynamicFields.push(field_name);
                            if (command.options) {
                                setDropdownOptions(field_name, command.data);
                            }
                            if (command.default_value != '') {
                                setElementValue(field_name, command.default_value);
                            }
                        }

                        if (command.callback) {
                            if (typeof window[command.callback] === "function")
                            {
                                window[command.callback](form, data, command);
                            }
                        }
                    }
                    if (userMeta !== undefined) {
                        var tempUserMeta = $.extend({}, userMeta);
                        for (var key in tempUserMeta) {
                            if (dynamicFields.indexOf(key) == -1) {
                                delete tempUserMeta[key];
                            }
                        }
                        prefilUsermeta(tempUserMeta, form); // Prefil data after loading dynamic fields
                    }
                    jQuery('body').trigger('erf_edit_submission_field', [form, field_name, submission]);
                }

            }).fail(function (xhr, textStatus, e) {
                //console.log(xhr.responseText);
            });
        }

        var getElementValue = function (name) {
            var fieldInstance = form.find('[name="' + name + '"]');
            var value = fieldInstance.val();
            return value;
        }

        var setElementValue = function (name, value) {
            var fieldInstance = form.find('[name="' + name + '"]');
            if (fieldInstance.is(':radio')) {
                fieldInstance.filter('[value="' + value + '"]').prop('checked', true);
            } else if (fieldInstance.is(':checkbox')) {
                if (value) {
                    if (value.constructor === Array) {
                        for (var i = 0; i < value.length; i++) {
                            fieldInstance.filter('[value="' + value[i] + '"]').prop('checked', true);
                        }
                    } else
                    {
                        fieldInstance.filter('[value="' + value + '"]').prop('checked', true);
                    }
                }

            } else {
                fieldInstance.val(value);
            }
            fieldInstance.trigger('change');
        }
        var setDropdownOptions = function (name, values) {
            var fieldInstance = form.find('[name="' + name + '"]');
            fieldInstance.empty();
            if (values.length == 0) {
                fieldInstance.editableSelect();
            } else
            {
                if (fieldInstance.hasClass('es-input')) {
                    fieldInstance.editableSelect('destroy'); // In case it is an editable select
                }
                var fieldInstance = form.find('[name="' + name + '"]');
                $.each(values, function (val, label) {
                    fieldInstance.append($('<option>', {
                        value: val,
                        text: label
                    }));
                });
            }
        }

        var fieldCommandOnChange = function (rules) {
            for (var i = 0; i < rules.length; i++) {
                var changeRule = rules[i];
                (function (changeRule) {
                    var data = {action: 'erforms_field_change_command', change_action: changeRule.action, field_name: changeRule.field_name, form_id: formId};
                    var fieldInstance = form.find('[name="' + changeRule.field_name + '"]');

                    if (fieldInstance.length == 0)
                        return;

                    fieldInstance.change(function () {
                        data.field_value = getElementValue(changeRule.field_name);
                        $.post(erf_global.ajax_url, data, function (res) {
                            if (res.success) {
                                var commands = res.data.commands;
                                var dynamicFields = [];
                                for (var i = 0; i < commands.length; i++) {
                                    var command = commands[i];
                                    for (var j = 0; j < command.on.length; j++) {
                                        var field_name = command.on[j];
                                        dynamicFields.push(field_name);
                                        if (command.options) {
                                            setDropdownOptions(field_name, command.data)
                                        }
                                        if (command.default_value != '') {

                                            setElementValue(field_name, command.default_value);
                                        }
                                    }
                                    if (command.callback) {
                                        if (typeof window[command.callback] === "function")
                                        {
                                            window[command.callback](form, fieldInstance, data, command);
                                        }
                                    }
                                    fieldInstance.parsley().removeError(fieldInstance.prop('name'));
                                    if (command.error) {
                                        fieldInstance.parsley().addError(fieldInstance.prop('name'), {message: command.error, updateClass: true});
                                        // alert(command.error);
                                    } else
                                    {
                                        fieldInstance.parsley().removeError(fieldInstance.prop('name'));
                                    }

                                }
                                if (userMeta !== undefined) {
                                    var tempUserMeta = $.extend({}, userMeta);
                                    for (var key in tempUserMeta) {
                                        if (dynamicFields.indexOf(key) == -1) {
                                            delete tempUserMeta[key];
                                        } else
                                        {
                                            delete userMeta[key];
                                        }
                                    }
                                    prefilUsermeta(tempUserMeta, form); // Prefil data after loading dynamic fields
                                }
                                jQuery('body').trigger('erf_edit_submission_field', [form, field_name, submission]);
                            }
                        }).fail(function (xhr, textStatus, e) {
                            //console.log(xhr.responseText);
                        });
                    });
                })(changeRule)
            }
        }
    });
    /*****************************Front form ends here*******************/
    
    /*****************************Edit submissions here ****************/
    $(document).on('erf_edit_submission_form', function (ev, form, submission, formData)
    {
        if (typeof form === "undefined" || !formData || !submission)
            return;
        var prepareForm = function () {
            if (!erf_global.is_admin) {
                form.find(':input,:radio,:checkbox').not('[type="submit"]').addClass('erf-disabled');  // By default disable all the fields
                if (formData.hasOwnProperty('en_edit_sub') && formData.hasOwnProperty('edit_fields') && formData.edit_fields.length > 0) {
                    for (var i = 0; i < formData.edit_fields.length; i++) {
                        var single = $("[name=" + formData.edit_fields[i] + "]"); // For single fields
                        var multi = $("[name='" + formData.edit_fields[i] + "[]']"); // For array type of fields
                        if (single.length > 0) {
                            single.removeClass('erf-disabled');
                        }

                        if (multi.length > 0) {
                            multi.removeClass('erf-disabled');
                        }
                    }
                }
            }
            $.each(submission.fields_data, function (key, field_data) {
                var field = $("[name=" + field_data.f_name + "]");
                if (field_data.f_type == 'checkbox-group' || field_data.f_type == 'select')
                {
                    if (field.length == 0)
                    {
                        field = $("[name='" + field_data.f_name + "[]']");
                    }
                }
                if (field.length == 0 /*|| field.prop('disabled')*/)
                    return;

                if (field.is(':radio') || field.is(':checkbox')) {
                    field.each(function () {
                        if (jQuery(this).val() == field_data.f_val) {
                            jQuery(this).prop('checked', true);
                            return;
                        }
                    });
                }
                if (field.is(':file')) { // Skip file type inputs
                    return;
                }
                if (!field.is(':radio')) { // Excluding radio inputs
                    field.val(field_data.f_val);
                }

                field.trigger('change');
            });
            $('<input>').attr({style: 'display:none;', type: 'text', name: 'submission_id', value: submission.id}).appendTo(form);
        }
        prepareForm();
    });
    $(document).on('erf_edit_submission_field', function (ev, form, fieldName, submission){
        if (!submission)
            return;
        $.each(submission.fields_data, function (key, field_data) {
            if (fieldName == field_data.f_name) {
                var field = $("[name=" + fieldName + "]");
                if (field_data.f_type == 'checkbox-group' || field_data.f_type == 'select')
                {
                    if (field.length == 0)
                    {
                        field = $("[name='" + field_data.f_name + "[]']");
                    }
                }
                if (field.length == 0)
                    return;
                if (field.is(':radio') || field.is(':checkbox')) {
                    field.each(function () {
                        if (jQuery(this).val() == field_data.f_val) {
                            jQuery(this).prop('checked', true);
                            return;
                        }
                    });
                }
                if (field.is(':file')) { // Skip file type inputs
                    return;
                }
                if (!field.is(':radio')) { // Excluding radio inputs
                    field.val(field_data.f_val);
                }
                field.trigger('change');
            }
        });

    });
    /*****************************Edit submissions ends here ****************/
    
    /*****************************Login Form ****************************/
    $(document).bind('erf_process_login_form',function(ev){
        $(".erf-show-register,.erf-show-login,.erf-show-lost-password").click(function () {
            var erfContainer = $(this).closest('.erf-container');
            var loginContainer = erfContainer.find('.erf-login-container');
            var registrationForm = erfContainer.find('.erf-reg-form-container');
            var lostPasswordContainer = erfContainer.find('.erf-password-lost-container');

            if (!erfContainer.hasClass('erf-login-register')) {
                if (registrationForm.length > 0)
                    registrationForm.slideUp();
            }
            if (loginContainer.length > 0)
                loginContainer.slideUp();

            if (lostPasswordContainer.length > 0)
                lostPasswordContainer.slideUp();

            if ($(this).hasClass('erf-show-register')) {
                registrationForm.slideDown();
            }

            if ($(this).hasClass('erf-show-login')) {
                loginContainer.slideDown();
            }

            if ($(this).hasClass('erf-show-lost-password')) {
                ;
                lostPasswordContainer.slideDown();
            }
        });

        $('.erf-reset-password').click(function () {
            var lostPasswordContainer = $(this).closest('.erf-password-lost-container');
            var erfContainer = $(this).closest('.erf-container');
            var loginContainer = erfContainer.find('.erf-login-container');
            lostPasswordContainer.find('.erf-error').html();
            submitButton = $(this);
            submitButton.append('<span class="erf-loader"></span>');
            $.ajax({
                url: erf_global.ajax_url,
                type: 'POST',
                data: {'user_login': lostPasswordContainer.find('#erf_user_login').val(), 'action': 'erf_reset_password'},
                success: function (response) {
                    try {
                        response = JSON.parse(response);
                        if (response.success) {
                            lostPasswordContainer.slideUp();
                            loginContainer.slideDown();
                            loginContainer.find('.erf-message').html(response.msg);
                            loginContainer.find('.erf-error').html('');
                            lostPasswordContainer.find('.erf-error').html('');
                        } else {
                            lostPasswordContainer.find('.erf-error').html(response.msg);
                        }
                        submitButton.find('.erf-loader').remove();
                    } catch (ex) {
                    }
                },
            });
        });

        $('.erf-login-form').submit(function (e) {
            var loginContainer = $(this).closest('.erf-login-container');
            var loginForm = $(this);
            var rememberme = '';
            if (loginForm.find('#erf_rememberme').is(':checked')) {
                rememberme = 'forever';
            }

            loginContainer.find('.erf-error').html('');
            var data = {
                'erf_username': loginForm.find('#erf_username').val(),
                'erf_password': loginForm.find('#erf_password').val(),
                'rememberme': rememberme,
                'erf_login_nonce': loginForm.find('#erf_login_nonce').val(),
                'action': 'erf_login_user'
            };
            var recap = loginForm.find('.erf-recaptcha');
            if (recap.length > 0) {
                data['g-recaptcha-response'] = loginForm.find('#g-recaptcha-response').val();
            }
            $.ajax({
                url: erf_global.ajax_url,
                type: 'POST',
                data: data,
                success: function (response) {
                    try {
                        response = JSON.parse(response);
                        if (response.success) {
                            if (response.hasOwnProperty('reload')) {
                                loginContainer.find("#erf_login_reload_form").submit();
                            } 
                            else if(response.hasOwnProperty('redirect')) {
                                $(location).attr('href',response.redirect);
                            }
                        } else {
                            if (recap.length > 0 && typeof grecaptcha !== 'undefined' && grecaptcha && grecaptcha.reset) {
                                grecaptcha.reset();
                            }
                            loginContainer.find('.erf-error').html(response.msg);
                        }

                    } catch (ex) {
                    }
                },
            });
            e.preventDefault();
        });

    });
    /*****************************Login Form ends here****************************/
    
    /*****************************My Account ****************************/
    $(document).bind('erf_process_my_account',function(ev){
        var windowWidth = $(window).width();
        var myAccountDiv = $('.erf-my-account');
        if(myAccountDiv.length==0)
            return;
        var autoResize = function () {
            if (windowWidth > 1024) {
                if (myAccountDiv.innerWidth() < 700) {
                    myAccountDiv.closest('.erf-container').addClass('erf-small-inner');
                }
            }
        };
        autoResize();
        $(window).resize(function () {
            autoResize();
        });
        $('.erf-load-submission-row').click(function () {
            var submissionID = $(this).data('submission-id');
            var formID = $(this).data('form-id');
            var submissionContainer = $(this).siblings('.erf-modal');
            var contentContainer= submissionContainer.find('.erf-modal-body');
            submissionContainer.toggle();
            contentContainer.html(erf_global.js_strings.loading_submission_info)
            // Fetch updated submission info.
            $.post(erf_global.ajax_url, {form_id: formID, submission_id: submissionID, action: 'erforms_get_submission_html'}, function (res) {
                if (res.success) {
                    if (res.data.hasOwnProperty('html')) {
                        contentContainer.html(res.data.html);
                    }
                }
            }).fail(function (xhr, textStatus, e) {
                contentContainer.html(erf_global.js_strings.edit_form_load_error);
            });
        });
        
        var registerCloseModal= function(){
            $('.erf-modal-close').click(function () {
                $(this).closest('.erf-modal').hide();
            });
        }
        registerCloseModal();
        // Edit submission                
        $('.erf-edit-submission-row').click(function () {
            var modalContainer = $(this).parent('.erf-my-account-col-edit').find('.erf-modal');
            var submissionContainer = modalContainer.find('.erf-modal-body');
            var submissionID = $(this).data('submission-id');
            var formID = $(this).data('form-id');
            modalContainer.show();
            submissionContainer.html(erf_global.js_strings.loading_edit_form);
            $.post(erf_global.ajax_url, {form_id: formID, submission_id: submissionID, action: 'erforms_get_form_for_edit'}, function (res) {
                if (res.success) {
                    if (res.data.hasOwnProperty('form_html')) {
                        submissionContainer.html(res.data.form_html);
                        $(document).trigger('erf_process_form', [modalContainer]);
                    }
                    $(document).trigger('erf_submission_edited', [formID,submissionID,res]);
                }
                else{
                    if(res.data.hasOwnProperty('error')){
                         submissionContainer.html(res.data.error);
                    }
                     $(document).trigger('erf_submission_edit_error', [formID,submissionID,res]);
                }
                registerCloseModal();
            }).fail(function (xhr, textStatus, e) {
                submissionContainer.html(erf_global.js_strings.edit_form_load_error);
            });
        });
        
        // Delete submission
        $('.erf-delete-submission-row').click(function () {
            var deletionConfirmed= confirm(erf_global.my_account.sub_del_confirm_msg);
            if(!deletionConfirmed)
                return;
            var submissionID = $(this).data('submission-id');
            var formID = $(this).data('form-id');
            var row= $(this).closest('.erf-my-account-details');
            $.post(erf_global.ajax_url, {form_id: formID, submission_id: submissionID, action: 'erforms_delete_submission'}, function (res) {
                if (res.success) {
                   row.remove();
                   $(document).trigger('erf_submission_deleted', [formID,submissionID,res]);
                }
                else{
                    if(res.data.hasOwnProperty('msg')){
                         alert(res.data.msg);
                         $(document).trigger('erf_submission_deletion_error', [formID,submissionID,res]);
                    }
                }
            }).fail(function (xhr, textStatus, e) {
                alert('Unable to connect to server');
            });
        });
        
        $('.erf-my-account-nav a').click(function () {
            $('.erf-my-account-nav a').parent().removeClass('erf-my-account-navigation-link-active');
            $(this).parent().addClass('erf-my-account-navigation-link-active');
            var tagid = $(this).data('tag');
            $('.erf-my-account-profile-tab').removeClass('active').addClass('erf-hidden');
            $('#' + tagid).addClass('active').removeClass('erf-hidden');
        });

        window.Parsley.addValidator('confirmPassword', {
            validateString: function (value, passwordFieldId) {
                return $("#" + passwordFieldId).val() == value;
            },
            messages: {
                en: erf_global.parsley.confirmPassword,
            }
        });


        $(".erf-change-password").submit(function(event){
            var form = $(this);
            var formContainer=$(form).closest('.erf-my-account-profile-tab');
            var formInstance = form.parsley();
            var formData = new FormData(form[0]);
            var errorContainer= formContainer.find('.erf-errors');
            errorContainer.html('');
            form.find('button[type="submit"]').append('<span class="erf-loader"></span>');
            
            $.ajax({
                url: erf_global.ajax_url,
                type: 'POST',
                data: formData,
                async: true,
                success: function (response) {
                         if(response.success){
                             var message= response.data.msg;
                             form.find('button[type="submit"] .erf-loader').remove();
                             formContainer.html(message);
                             
                         }
                         else
                         {  
                            form.find('button[type="submit"] .erf-loader').remove();
                            if(response.data.hasOwnProperty('errors')){
                                for(var i=0;i<response.data.errors.length;i++){
                                    errorContainer.append('<div class="erf-error-row">' + response.data.errors[i] + '</div><br>');
                                }
                            }
                             
                         }
                },
                cache: false,
                contentType: false,
                processData: false
            });
            event.preventDefault();
        });
        $('.erf-password').password(erf_global.my_account.pass_config);
        $(".erf-print-submission").click(function(){
            $(this).closest('.erf-submission-info').find('.erf-modal-body').printThis();
        });
    });
    $(window).bind("load", function() {
        var currentURL = $(location).attr('href');
        if(currentURL.indexOf('erf_paged') != -1){
            $('a[data-tag="submissions"]').trigger( "click" );
        }
    });
    /*****************************My Account ends here ****************************/
    
    /****************************Common*******************************************/
    $(document).bind('erf_register_common_scripts',function(ev){
        $(".erf_resend_ver_link").click(function(){
            var tar= $(this);
            var container= tar.closest('.erf-container');
            $.ajax({
                url: erf_global.ajax_url,
                type: 'POST',
                data: {nonce: $(this).data('nonce'),sub: $(this).data('sub'), 'action': 'erf_resend_verification'},
                success: function (response) {
                    try {
                        if (response.success) {
                            erf_popup_message({content: response.data.msg},container);
                        } else {
                            erf_popup_message({content: response.data.msg},container);
                        };
                    } catch (ex) {
                    }
                    tar.removeClass('erf-text-spinner');
                },
                beforeSend:function(){
                    tar.addClass('erf-text-spinner');
                }
            });
        });
    });
})(jQuery);

/*Form Ends here*/

jQuery(document).ready(function(){
   erf_global_triggers();
});


